# Generated by roxygen2: do not edit by hand

S3method(coef,tidyFit)
S3method(coef,tidyfit.models)
S3method(explain,tidyFit)
S3method(explain,tidyfit.models)
S3method(fitted,tidyFit)
S3method(fitted,tidyfit.models)
S3method(glance,tidyFit)
S3method(plot,tidyFit)
S3method(predict,tidyFit)
S3method(predict,tidyfit.models)
S3method(resid,tidyFit)
S3method(residuals,tidyfit.models)
S3method(summary,tidyFit)
S3method(tidy,tidyFit)
S3method(unnest,tidyfit.models)
export(classify)
export(explain)
export(get_model)
export(get_tidyFit)
export(m)
export(regress)
importFrom(MASS,rlm)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(crayon,bold)
importFrom(crayon,green)
importFrom(crayon,italic)
importFrom(crayon,red)
importFrom(crayon,yellow)
importFrom(dials,grid_regular)
importFrom(dials,mixture)
importFrom(dials,penalty)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,coalesce)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_nest)
importFrom(dplyr,group_split)
importFrom(dplyr,group_vars)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,tibble)
importFrom(dplyr,ungroup)
importFrom(furrr,furrr_options)
importFrom(furrr,future_pmap_dfr)
importFrom(generics,explain)
importFrom(methods,formalArgs)
importFrom(progressr,progressor)
importFrom(purrr,cross)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_dbl)
importFrom(purrr,map_dfr)
importFrom(purrr,partial)
importFrom(purrr,quietly)
importFrom(purrr,safely)
importFrom(purrr,transpose)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rsample,int_pctl)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,cor.test)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(tibble,enframe)
importFrom(tibble,new_tibble)
importFrom(tidyr,complete)
importFrom(tidyr,expand_grid)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,spread)
importFrom(tidyr,unnest)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
importFrom(utils,installed.packages)
importFrom(utils,object.size)
importFrom(vctrs,vec_as_names)
importFrom(yardstick,mn_log_loss)
importFrom(yardstick,rmse)
